using System;

namespace TuxHunter.Controllers
{
	/// <summary>
	/// Controller for the Geek. This d00d just walks back and forth all the time.
	/// </summary>
	public class GeekController : RandomBehaviorController
	{
		public GeekController()
		{
			this.AddSubController(MakeWalkController(), 0.5);
		}

		private AbstractSubController MakeWalkController()
		{
			SequenceController result = new SequenceController();

			result.AddSubController(new MoveLeftController(30));
			result.AddSubController(new MoveRightController(60));
            result.AddSubController(new DoNothingController(30));
            result.AddSubController(new MoveLeftController(45));
            result.AddSubController(new MoveRightController(30));
            result.AddSubController(new DoNothingController(40));
            result.AddSubController(new MoveLeftController(15));
            result.AddSubController(new DoNothingController(45));
            result.AddSubController(new MoveRightController(30));
			result.AddSubController(new MoveLeftController(30));

			return result;
		}
	}
}
